==> skrooge_0.6.0.orig.tar.gz: skrooge-0.6.0/skrooge_categories/skgcategoriespluginwidget.cpp <==
/***************************************************************************
 *   Copyright (C) 2008 by S. MANKOWSKI / G. DE BURE skrooge@mankowski.fr  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>  *
 ***************************************************************************/
/** @file
 * This file is Skrooge plugin to generate categories.
 *
 * @author Stephane MANKOWSKI / Guillaume DE BURE
 */
#include "skgcategoriespluginwidget.h"
#include "skgmainpanel.h"
#include "skgtraces.h"
#include "skgobjectmodel.h"
#include "skgsortfilterproxymodel.h"
#include "skgdocumentbank.h"
#include "skgcategoryobject.h"
#include "skgtransactionmng.h"

#include <klocale.h>

#include <QDomDocument>
#include <QtGui/QWidget>
#include <QGraphicsScene>
#include <QHeaderView>
#include <QKeyEvent>

SKGCategoriesPluginWidget::SKGCategoriesPluginWidget ( SKGDocumentBank* iDocument )
        : SKGTabPage ( iDocument )
{
    SKGTRACEIN ( 10, "SKGCategoriesPluginWidget::SKGCategoriesPluginWidget" );

    ui.setupUi ( this );

    ui.kAddCategoryButton->setIcon ( KIcon ( "list-add" ) );
    ui.kModifyCategoryButton->setIcon ( KIcon ( "dialog-ok-apply" ) );
    ui.kDeleteUnusedButton->setIcon ( KIcon ( "edit-delete" ) );

    objectModel = new SKGObjectModel ( ( SKGDocumentBank* ) getDocument(), "v_category_display", "1=0", this, "r_category_id", false );
    SKGSortFilterProxyModel* modelproxy = new SKGSortFilterProxyModel ( this );
    modelproxy->setSourceModel ( objectModel );
    modelproxy->setSortRole ( Qt::UserRole );
    modelproxy->setDynamicSortFilter ( true );
    ui.kCategoriesView->setModel ( modelproxy );

    //Add registered global action in contextual menu
    if ( SKGMainPanel::getMainPanel() )
    {
        ui.kCategoriesView->insertAction ( 0, SKGMainPanel::getMainPanel()->getGlobalAction ( "edit_delete" ) );
        QAction* sep=new QAction ( this );
        sep->setSeparator ( true );
        ui.kCategoriesView->insertAction ( 0, sep );
        ui.kCategoriesView->insertAction ( 0, SKGMainPanel::getMainPanel()->getGlobalAction ( "open_report" ) );

        ui.kCategoriesView->resizeColumnToContents ( 0 );
        ui.kCategoriesView->header()->setStretchLastSection ( false );
    }

    ui.kTitle->setPixmap ( KIcon ( "dialog-information" ).pixmap ( 22, 22 ), KTitleWidget::ImageLeft );

    connect ( ( const QObject* ) getDocument(), SIGNAL ( tableModified ( QString, int ) ), this, SLOT ( dataModified ( QString, int ) ), Qt::QueuedConnection );
    dataModified ( "", 0 );

    connect ( ui.kCategoriesView, SIGNAL ( selectionChangedDelayed() ), this, SLOT ( onSelectionChanged() ) );

    connect ( objectModel, SIGNAL ( beforeReset() ), ui.kCategoriesView, SLOT ( saveSelection() ) );
    connect ( objectModel, SIGNAL ( afterReset() ), ui.kCategoriesView, SLOT ( resetSelection() ) );

    //Set Event filters to catch CTRL+ENTER or SHIFT+ENTER
    this->installEventFilter ( this );
}

SKGCategoriesPluginWidget::~SKGCategoriesPluginWidget()
{
    SKGTRACEIN ( 10, "SKGCategoriesPluginWidget::~SKGCategoriesPluginWidget" );
    objectModel=NULL;
}

bool SKGCategoriesPluginWidget::eventFilter ( QObject *object, QEvent *event )
{
    if ( event && event->type() == QEvent::KeyPress )
    {
        QKeyEvent *keyEvent = ( QKeyEvent * ) event;
        if ( ( keyEvent->key() == Qt::Key_Return || keyEvent->key() == Qt::Key_Enter ) && object==this )
        {
            if ( QApplication::keyboardModifiers() &Qt::ControlModifier && ui.kAddCategoryButton->isEnabled() )
            {
                ui.kAddCategoryButton->click();
            }
            else if ( QApplication::keyboardModifiers() &Qt::ShiftModifier && ui.kModifyCategoryButton->isEnabled() )
            {
                ui.kModifyCategoryButton->click();
            }
        }
    }

    return false;
}

void SKGCategoriesPluginWidget::dataModified ( const QString& iTableName, int iIdTransaction )
{
    SKGTRACEIN ( 10, "SKGCategoriesPluginWidget::dataModified" );
    Q_UNUSED ( iIdTransaction );

    if ( iTableName=="category" || iTableName.isEmpty() )
    {
        //Correction bug 2299394 vvv
        if ( ui.kCategoriesView->isAutoResized() ) ui.kCategoriesView->resizeColumnsToContents();
        //Correction bug 2299394 ^^^
        onEditorModified();
    }
}

void SKGCategoriesPluginWidget::onSelectionChanged()
{
    SKGTRACEIN ( 10, "SKGCategoriesPluginWidget::onSelectionChanged" );

    SKGObjectBase::SKGListSKGObjectBase objs=getSelectedObjects();
    if ( objs.count() )
    {
        SKGCategoryObject cat=objs.at ( 0 );
        ui.kNameInput->setText ( cat.getName() );
    }

    onEditorModified();
    emit selectionChanged();
}

SKGObjectBase::SKGListSKGObjectBase SKGCategoriesPluginWidget::getSelectedObjects()
{
    return ui.kCategoriesView->getSelectedObjects();
}

int SKGCategoriesPluginWidget::getNbSelectedObjects()
{
    return ui.kCategoriesView->getNbSelectedObjects();
}

QString SKGCategoriesPluginWidget::getState()
{
    SKGTRACEIN ( 10, "SKGCategoriesPluginWidget::getState" );
    QDomDocument doc ( "SKGML" );
    QDomElement root = doc.createElement ( "parameters" );
    doc.appendChild ( root );

    //Memorize table settings
    root.setAttribute ( "view", ui.kCategoriesView->getState() );
    return doc.toString();
}

void SKGCategoriesPluginWidget::setState ( const QString& iState )
{
    SKGTRACEIN ( 10, "SKGCategoriesPluginWidget::setState" );
    QDomDocument doc ( "SKGML" );
    doc.setContent ( iState );
    QDomElement root = doc.documentElement();

    if ( objectModel )
    {
        objectModel->setFilter ( "1=1" );
    }

    //!!! Must be done here after setFilter
    ui.kCategoriesView->setState ( root.attribute ( "view" ) );

    //Correction bug 2299394 vvv
    if ( ui.kCategoriesView->isAutoResized() ) ui.kCategoriesView->resizeColumnsToContents();
    //Correction bug 2299394 ^^^
}

QString SKGCategoriesPluginWidget::getDefaultStateAttribute()
{
    return "SKGCATEGORIES_DEFAULT_PARAMETERS";
}

QWidget* SKGCategoriesPluginWidget::getWidgetForPrint()
{
    return ui.kCategoriesView;
}

void SKGCategoriesPluginWidget::onEditorModified()
{
    _SKGTRACEIN ( 10, "SKGCategoriesPluginWidget::onEditorModified" );
    int nb=getNbSelectedObjects();
    ui.kModifyCategoryButton->setEnabled ( !ui.kNameInput->text().isEmpty() && nb==1 );
    ui.kAddCategoryButton->setEnabled ( !ui.kNameInput->text().isEmpty() && nb<=1 );
}

void SKGCategoriesPluginWidget::onAddCategory()
{
    SKGError err;
    _SKGTRACEINRC ( 10, "SKGCategoriesPluginWidget::onAddCategory", err );
    SKGCategoryObject cat;
    QString name=ui.kNameInput->text();
    QApplication::setOverrideCursor ( QCursor ( Qt::WaitCursor ) );
    {
        //Get Selection
        SKGObjectBase::SKGListSKGObjectBase selection=getSelectedObjects();

        int nb=selection.count();
        SKGBEGINTRANSACTION ( *getDocument(), i18nc ( "Noun, name of the user action", "Category creation [%1]", name )  , err );

        err=SKGCategoryObject::createPathCategory ( ( const SKGDocumentBank* ) getDocument(), name, cat );
        if ( nb==1 && err.isSucceeded() )
        {
            SKGCategoryObject parent=selection[0];
            SKGCategoryObject rootOfBranch;
            err=cat.getRootCategory ( rootOfBranch );
            if ( err.isSucceeded() ) err=rootOfBranch.setParentCategory ( parent );
            if ( err.isSucceeded() ) err=rootOfBranch.save();
        }
    }
    QApplication::restoreOverrideCursor();

    //status bar
    if ( err.isSucceeded() )
    {
        ui.kCategoriesView->selectObject ( cat.getUniqueID() );
        err=SKGError ( 0, i18nc ( "Successful message after an user action", "Category [%1] created", name ) );
    }
    else err.addError ( ERR_FAIL, i18nc ( "Error message", "Category creation failed" ) );

    //Display error
    SKGMainPanel::getMainPanel()->displayErrorMessage ( err );
}

void SKGCategoriesPluginWidget::onUpdateCategory()
{
    SKGError err;
    _SKGTRACEINRC ( 10, "SKGCategoriesPluginWidget::onUpdateCategory", err );
    QApplication::setOverrideCursor ( QCursor ( Qt::WaitCursor ) );
    //Get Selection
    SKGObjectBase::SKGListSKGObjectBase selection=getSelectedObjects();

    int nb=selection.count();
    if ( nb==1 )
    {
        SKGBEGINTRANSACTION ( *getDocument(), i18nc ( "Noun, name of the user action", "Category [%1] update", ui.kNameInput->text() )  , err );

        //Modification of category object
        SKGCategoryObject cat=selection[0];
        err=cat.setName ( ui.kNameInput->text() );
        if ( err.isSucceeded() ) err=cat.save();

        //status bar
        if ( err.isSucceeded() ) err=SKGError ( 0, i18nc ( "Successful message after an user action", "Category [%1] updated", ui.kNameInput->text() ) );
        else err.addError ( ERR_FAIL, i18nc ( "Error message", "Category update failed" ) );
    }
    QApplication::restoreOverrideCursor();

    //Display error
    SKGMainPanel::getMainPanel()->displayErrorMessage ( err );
}

void SKGCategoriesPluginWidget::onDeleteUnused()
{
    SKGError err;
    _SKGTRACEINRC ( 10, "SKGCategoriesPluginWidget::onDeleteUnused", err );
    QApplication::setOverrideCursor ( QCursor ( Qt::WaitCursor ) );
    {
        SKGBEGINTRANSACTION ( *getDocument(), i18nc ( "Noun, name of the user action", "Delete unused categories" )  , err );

        //Modification of category object
        QString sql="DELETE FROM category WHERE id IN (SELECT id FROM v_category_display WHERE i_SUMNBOPERATIONS=0)";
        err=SKGServices::executeSqliteOrder ( getDocument(), sql );

        //status bar
        if ( err.isSucceeded() ) err=SKGError ( 0, i18nc ( "Successful message after an user action", "Unused categories deleted" ) );
        else err.addError ( ERR_FAIL, i18nc ( "Error message", "Unused categories deletion failed" ) );
    }
    QApplication::restoreOverrideCursor();

    //Display error
    SKGMainPanel::getMainPanel()->displayErrorMessage ( err );
}

void SKGCategoriesPluginWidget::onDoubleClick()
{
    _SKGTRACEIN ( 10, "SKGCategoriesPluginWidget::onDoubleClick" );

    //Get selection
    SKGObjectBase::SKGListSKGObjectBase selection=getSelectedObjects();
    if ( selection.count() ==1 )
    {
        SKGCategoryObject cat ( selection.at ( 0 ) );
        QString name=cat.getFullName();

        //Build where clause and title
        QString wc="t_REALCATEGORY";
        QString title;
        if ( name.isEmpty() )
        {
            wc+=" IS NULL OR t_REALCATEGORY=''";
            wc='('+wc+')';
            title=i18nc ( "Noun, a list of items", "Sub operations with empty category" );
        }
        else
        {
            wc+=" = '"+SKGServices::stringToSqlString ( name ) +"' OR t_REALCATEGORY like '"+SKGServices::stringToSqlString ( name ) +OBJECTSEPARATOR+"%'";
            wc='('+wc+')';
            title=i18nc ( "Noun, a list of items", "Sub operations with category containing '%1'",  name );
        }

        //Open
        if ( QApplication::keyboardModifiers() &Qt::ControlModifier && QApplication::keyboardModifiers() &Qt::ShiftModifier )
        {
            //Call debug plugin
            QDomDocument doc ( "SKGML" );
            QDomElement root = doc.createElement ( "parameters" );
            doc.appendChild ( root );
            root.setAttribute ( "sqlOrder", "SELECT * from v_operation_consolidated WHERE "+wc );

            SKGMainPanel::getMainPanel()->setNewTabContent ( SKGMainPanel::getMainPanel()->getPluginByName ( "Skrooge debug plugin" ), -1, doc.toString() );
        }
        else
        {
            //Call operation plugin
            QDomDocument doc ( "SKGML" );
            doc.setContent ( getDocument()->getParameter ( "SKGOPERATION_CONSOLIDATED_DEFAULT_PARAMETERS" ) );
            QDomElement root = doc.documentElement();
            if ( root.isNull() )
            {
                root=doc.createElement ( "parameters" );
                doc.appendChild ( root );
            }

            root.setAttribute ( "operationTable", "v_operation_consolidated" );
            root.setAttribute ( "operationWhereClause", wc );
            root.setAttribute ( "title", title );
            root.setAttribute ( "title_icon", "skrooge_category" );
            root.setAttribute ( "currentPage", "-1" );

            SKGMainPanel::getMainPanel()->setNewTabContent ( SKGMainPanel::getMainPanel()->getPluginByName ( "Skrooge operation plugin" ), -1, doc.toString() );
        }
    }
}

void SKGCategoriesPluginWidget::cleanEditor()
{
    if ( getNbSelectedObjects() ==0 )
    {
        ui.kNameInput->setText ( "" );
    }
}

#include "skgcategoriespluginwidget.moc"

==> skrooge_0.7.2.2.orig.tar.gz: skrooge-0.7.2.2/skrooge_categories/skgcategoriespluginwidget.cpp <==
/***************************************************************************
 *   Copyright (C) 2008 by S. MANKOWSKI / G. DE BURE skrooge@mankowski.fr  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>  *
 ***************************************************************************/
/** @file
 * This file is Skrooge plugin to generate categories.
 *
 * @author Stephane MANKOWSKI / Guillaume DE BURE
 */
#include "skgcategoriespluginwidget.h"
#include "skgmainpanel.h"
#include "skgtraces.h"
#include "skgobjectmodel.h"
#include "skgsortfilterproxymodel.h"
#include "skgdocumentbank.h"
#include "skgcategoryobject.h"
#include "skgtransactionmng.h"

#include <klocale.h>

#include <QDomDocument>
#include <QtGui/QWidget>
#include <QGraphicsScene>
#include <QHeaderView>
#include <QKeyEvent>

SKGCategoriesPluginWidget::SKGCategoriesPluginWidget ( SKGDocumentBank* iDocument )
        : SKGTabPage ( iDocument )
{
    SKGTRACEIN ( 10, "SKGCategoriesPluginWidget::SKGCategoriesPluginWidget" );

    ui.setupUi ( this );

    ui.kAddCategoryButton->setIcon ( KIcon ( "list-add" ) );
    ui.kModifyCategoryButton->setIcon ( KIcon ( "dialog-ok-apply" ) );
    ui.kDeleteUnusedButton->setIcon ( KIcon ( "edit-delete" ) );

    objectModel = new SKGObjectModel ( static_cast<SKGDocumentBank*>(getDocument()), "v_category_display", "1=0", this, "r_category_id", false );
    SKGSortFilterProxyModel* modelproxy = new SKGSortFilterProxyModel ( this );
    modelproxy->setSourceModel ( objectModel );
    modelproxy->setSortRole ( Qt::UserRole );
    modelproxy->setDynamicSortFilter ( true );
    ui.kCategoriesView->setModel ( modelproxy );

    //Add registered global action in contextual menu
    if ( SKGMainPanel::getMainPanel() )
    {
        ui.kCategoriesView->insertGlobalAction ( "edit_delete" );
        ui.kCategoriesView->insertGlobalAction ();
        ui.kCategoriesView->insertGlobalAction ( "edit_find" );
        ui.kCategoriesView->insertGlobalAction ( "open_report" );

        ui.kCategoriesView->resizeColumnToContents ( 0 );
        ui.kCategoriesView->header()->setStretchLastSection ( false );
    }

    ui.kTitle->setPixmap ( KIcon ( "dialog-information" ).pixmap ( 22, 22 ), KTitleWidget::ImageLeft );

    connect ( ( const QObject* ) getDocument(), SIGNAL ( tableModified ( QString, int ) ), this, SLOT ( dataModified ( QString, int ) ), Qt::QueuedConnection );
    dataModified ( "", 0 );

    connect ( ui.kCategoriesView, SIGNAL ( selectionChangedDelayed() ), this, SLOT ( onSelectionChanged() ) );

    connect ( objectModel, SIGNAL ( beforeReset() ), ui.kCategoriesView, SLOT ( saveSelection() ) );
    connect ( objectModel, SIGNAL ( afterReset() ), ui.kCategoriesView, SLOT ( resetSelection() ) );

    //Set Event filters to catch CTRL+ENTER or SHIFT+ENTER
    this->installEventFilter ( this );
}

SKGCategoriesPluginWidget::~SKGCategoriesPluginWidget()
{
    SKGTRACEIN ( 10, "SKGCategoriesPluginWidget::~SKGCategoriesPluginWidget" );
    objectModel=NULL;
}

bool SKGCategoriesPluginWidget::eventFilter ( QObject *object, QEvent *event )
{
    if ( event && event->type() == QEvent::KeyPress )
    {
        QKeyEvent *keyEvent = ( QKeyEvent * ) event;
        if ( ( keyEvent->key() == Qt::Key_Return || keyEvent->key() == Qt::Key_Enter ) && object==this )
        {
            if ( QApplication::keyboardModifiers() &Qt::ControlModifier && ui.kAddCategoryButton->isEnabled() )
            {
                ui.kAddCategoryButton->click();
            }
            else if ( QApplication::keyboardModifiers() &Qt::ShiftModifier && ui.kModifyCategoryButton->isEnabled() )
            {
                ui.kModifyCategoryButton->click();
            }
        }
    }

    return false;
}

void SKGCategoriesPluginWidget::dataModified ( const QString& iTableName, int iIdTransaction )
{
    SKGTRACEIN ( 10, "SKGCategoriesPluginWidget::dataModified" );
    Q_UNUSED ( iIdTransaction );

    if ( iTableName=="category" || iTableName.isEmpty() )
    {
        //Correction bug 2299394 vvv
        if ( ui.kCategoriesView->isAutoResized() ) ui.kCategoriesView->resizeColumnsToContentsDelayed();
        //Correction bug 2299394 ^^^
        onEditorModified();
    }
}

void SKGCategoriesPluginWidget::onSelectionChanged()
{
    SKGTRACEIN ( 10, "SKGCategoriesPluginWidget::onSelectionChanged" );

    SKGObjectBase::SKGListSKGObjectBase objs=getSelectedObjects();
    if ( objs.count() )
    {
        SKGCategoryObject cat=objs.at ( 0 );
        ui.kNameInput->setText ( cat.getName() );
    }

    onEditorModified();
    emit selectionChanged();
}

SKGObjectBase::SKGListSKGObjectBase SKGCategoriesPluginWidget::getSelectedObjects()
{
    return ui.kCategoriesView->getSelectedObjects();
}

int SKGCategoriesPluginWidget::getNbSelectedObjects()
{
    return ui.kCategoriesView->getNbSelectedObjects();
}

QString SKGCategoriesPluginWidget::getState()
{
    SKGTRACEIN ( 10, "SKGCategoriesPluginWidget::getState" );
    QDomDocument doc ( "SKGML" );
    QDomElement root = doc.createElement ( "parameters" );
    doc.appendChild ( root );

    //Memorize table settings
    root.setAttribute ( "view", ui.kCategoriesView->getState() );
    return doc.toString();
}

void SKGCategoriesPluginWidget::setState ( const QString& iState )
{
    SKGTRACEIN ( 10, "SKGCategoriesPluginWidget::setState" );
    QDomDocument doc ( "SKGML" );
    doc.setContent ( iState );
    QDomElement root = doc.documentElement();

    if ( objectModel )
    {
        objectModel->setFilter ( "1=1" );
    }

    //!!! Must be done here after setFilter
    ui.kCategoriesView->setState ( root.attribute ( "view" ) );

    //Correction bug 2299394 vvv
    if ( ui.kCategoriesView->isAutoResized() ) ui.kCategoriesView->resizeColumnsToContentsDelayed();
    //Correction bug 2299394 ^^^
}

QString SKGCategoriesPluginWidget::getDefaultStateAttribute()
{
    return "SKGCATEGORIES_DEFAULT_PARAMETERS";
}

QWidget* SKGCategoriesPluginWidget::getWidgetForPrint()
{
    return ui.kCategoriesView;
}

void SKGCategoriesPluginWidget::onEditorModified()
{
    _SKGTRACEIN ( 10, "SKGCategoriesPluginWidget::onEditorModified" );
    int nb=getNbSelectedObjects();
    ui.kModifyCategoryButton->setEnabled ( !ui.kNameInput->text().isEmpty() && nb==1 );
    ui.kAddCategoryButton->setEnabled ( !ui.kNameInput->text().isEmpty() && nb<=1 );
}

void SKGCategoriesPluginWidget::onAddCategory()
{
    SKGError err;
    _SKGTRACEINRC ( 10, "SKGCategoriesPluginWidget::onAddCategory", err );
    SKGCategoryObject cat;
    QString name=ui.kNameInput->text();
    QApplication::setOverrideCursor ( QCursor ( Qt::WaitCursor ) );
    {
        //Get Selection
        SKGObjectBase::SKGListSKGObjectBase selection=getSelectedObjects();

        int nb=selection.count();
        SKGBEGINTRANSACTION ( *getDocument(), i18nc ( "Noun, name of the user action", "Category creation '%1'", name )  , err );

        if ( nb==1)
        {
            SKGCategoryObject parent=selection[0];
            name=parent.getFullName()+OBJECTSEPARATOR+name;
        }
        err=SKGCategoryObject::createPathCategory ( static_cast<SKGDocumentBank*>(getDocument()), name, cat );

    }
    QApplication::restoreOverrideCursor();

    //status bar
    if ( err.isSucceeded() )
    {
        ui.kCategoriesView->selectObject ( cat.getUniqueID() );
        err=SKGError ( 0, i18nc ( "Successful message after an user action", "Category '%1' created", name ) );
    }
    else err.addError ( ERR_FAIL, i18nc ( "Error message", "Category creation failed" ) );

    //Display error
    SKGMainPanel::getMainPanel()->displayErrorMessage ( err );
}

void SKGCategoriesPluginWidget::onUpdateCategory()
{
    SKGError err;
    _SKGTRACEINRC ( 10, "SKGCategoriesPluginWidget::onUpdateCategory", err );
    QApplication::setOverrideCursor ( QCursor ( Qt::WaitCursor ) );
    //Get Selection
    SKGObjectBase::SKGListSKGObjectBase selection=getSelectedObjects();

    int nb=selection.count();
    if ( nb==1 )
    {
        SKGBEGINTRANSACTION ( *getDocument(), i18nc ( "Noun, name of the user action", "Category '%1' update", ui.kNameInput->text() )  , err );

        //Modification of category object
        SKGCategoryObject cat=selection[0];
        err=cat.setName ( ui.kNameInput->text() );
        if ( err.isSucceeded() ) err=cat.save();

        //status bar
        if ( err.isSucceeded() ) err=SKGError ( 0, i18nc ( "Successful message after an user action", "Category '%1' updated", ui.kNameInput->text() ) );
        else err.addError ( ERR_FAIL, i18nc ( "Error message", "Category update failed" ) );
    }
    QApplication::restoreOverrideCursor();

    //Display error
    SKGMainPanel::getMainPanel()->displayErrorMessage ( err );
}

void SKGCategoriesPluginWidget::onDeleteUnused()
{
    SKGError err;
    _SKGTRACEINRC ( 10, "SKGCategoriesPluginWidget::onDeleteUnused", err );
    QApplication::setOverrideCursor ( QCursor ( Qt::WaitCursor ) );
    {
        SKGBEGINTRANSACTION ( *getDocument(), i18nc ( "Noun, name of the user action", "Delete unused categories" )  , err );

        //Modification of category object
        QString sql="DELETE FROM category WHERE id IN (SELECT id FROM v_category_display WHERE i_SUMNBOPERATIONS=0)";
        err=SKGServices::executeSqliteOrder ( getDocument(), sql );

        //status bar
        if ( err.isSucceeded() ) err=SKGError ( 0, i18nc ( "Successful message after an user action", "Unused categories deleted" ) );
        else err.addError ( ERR_FAIL, i18nc ( "Error message", "Unused categories deletion failed" ) );
    }
    QApplication::restoreOverrideCursor();

    //Display error
    SKGMainPanel::getMainPanel()->displayErrorMessage ( err );
}

void SKGCategoriesPluginWidget::onDoubleClick()
{
    _SKGTRACEIN ( 10, "SKGCategoriesPluginWidget::onDoubleClick" );

    //Get selection
    SKGObjectBase::SKGListSKGObjectBase selection=getSelectedObjects();
    if ( selection.count() ==1 )
    {
        SKGCategoryObject cat ( selection.at ( 0 ) );
        QString name=cat.getFullName();

        //Build where clause and title
        QString wc="t_REALCATEGORY";
        QString title;
        if ( name.isEmpty() )
        {
            wc+=" IS NULL OR t_REALCATEGORY=''";
            wc='('+wc+')';
            title=i18nc ( "Noun, a list of items", "Sub operations with empty category" );
        }
        else
        {
            wc+=" = '"+SKGServices::stringToSqlString ( name ) +"' OR t_REALCATEGORY like '"+SKGServices::stringToSqlString ( name ) +OBJECTSEPARATOR+"%'";
            wc='('+wc+')';
            title=i18nc ( "Noun, a list of items", "Sub operations with category containing '%1'",  name );
        }

        //Open
        if ( QApplication::keyboardModifiers() &Qt::ControlModifier && QApplication::keyboardModifiers() &Qt::ShiftModifier )
        {
            //Call debug plugin
            QDomDocument doc ( "SKGML" );
            QDomElement root = doc.createElement ( "parameters" );
            doc.appendChild ( root );
            root.setAttribute ( "sqlOrder", "SELECT * from v_operation_consolidated WHERE "+wc );

            SKGMainPanel::getMainPanel()->openPage ( SKGMainPanel::getMainPanel()->getPluginByName ( "Debug plugin" ), -1, doc.toString() );
        }
        else
        {
            //Call operation plugin
            QDomDocument doc ( "SKGML" );
            doc.setContent ( getDocument()->getParameter ( "SKGOPERATION_CONSOLIDATED_DEFAULT_PARAMETERS" ) );
            QDomElement root = doc.documentElement();
            if ( root.isNull() )
            {
                root=doc.createElement ( "parameters" );
                doc.appendChild ( root );
            }

            root.setAttribute ( "operationTable", "v_operation_consolidated" );
            root.setAttribute ( "operationWhereClause", wc );
            root.setAttribute ( "title", title );
            root.setAttribute ( "title_icon", "skrooge_category" );
            root.setAttribute ( "currentPage", "-1" );

            SKGMainPanel::getMainPanel()->openPage ( SKGMainPanel::getMainPanel()->getPluginByName ( "Skrooge operation plugin" ), -1, doc.toString() );
        }
    }
}

void SKGCategoriesPluginWidget::cleanEditor()
{
    if ( getNbSelectedObjects() ==0 )
    {
        ui.kNameInput->setText ( "" );
    }
}
void SKGCategoriesPluginWidget::activateEditor()
{
    ui.kNameInput->setFocus();
}

bool SKGCategoriesPluginWidget::isEditor()
{
    return true;
}

QWidget* SKGCategoriesPluginWidget::zoomableWidget() const
{
    return ui.kCategoriesView;
}
#include "skgcategoriespluginwidget.moc"
